
#ifndef BOOST_PROCESS_POSIX_EXECUTOR_HPP
#define BOOST_PROCESS_POSIX_EXECUTOR_HPP

#include <boost/process/config.hpp>
#include <boost/filesystem/path.hpp>
#include <boost/fusion/algorithm/iteration/for_each.hpp>
#include <boost/process/posix/initializers/initializer.hpp>

#include <fstream>
#include <vector>
#include <sys/types.h>

//////////////////////////////////////////////////////////////////
#define BOOST_PROCESS_EXECUTOR_METHOD(fnc)                        \
                                                                  \
template<typename INITIALIZERS> void fnc(const INITIALIZERS& izs) \
{                                                                 \
    boost::fusion::for_each(izs, initializer::lazy::fnc(*this));  \
}
//////////////////////////////////////////////////////////////////

namespace boost { namespace process { namespace posix {

    struct executor 
    {
        typedef boost::filesystem::path      path;
        typedef char                    char_type;
        typedef pid_t                    pid_type;

        executor() : m_exe(), m_arg_ptrs(), m_env_vars_ptrs(0), m_pid(-1), m_errno(0) {}
        
        template<typename INITIALIZERS> void exec(const INITIALIZERS& izs)
        {
            pre_fork_parent(izs);
            
            switch(m_pid = fork())
            {
                case  0:   post_fork_child (izs); return; 
                default:   post_fork_parent(izs); return; 
                case -1: failed_fork_parent(izs); return;
            }
        }

        template<typename INITIALIZERS> void post_fork_child(const INITIALIZERS& izs)
        {
            boost::fusion::for_each(izs, initializer::lazy::post_fork_child(*this));
        
            //execv("/xApplications/TextEdit.app/Contents/MacOS/TextEdit", 0);//m_arg_ptrs.ptr());//m_env_vars_ptrs); // doesn't return on success
            execve(m_exe.c_str(), m_arg_ptrs.ptr(), m_env_vars_ptrs); // doesn't return on success
            
            failed_exec_child(izs);
        }
        
        BOOST_PROCESS_EXECUTOR_METHOD(   pre_fork_parent)
        BOOST_PROCESS_EXECUTOR_METHOD(  post_fork_parent)
        BOOST_PROCESS_EXECUTOR_METHOD(failed_fork_parent)
        BOOST_PROCESS_EXECUTOR_METHOD(failed_exec_child )
    
        struct arg_ptrs
        {
            arg_ptrs() : m_arg_ptrs(2, (char*)0) {} // front is for app name, back is trailing null 
        
            void assign_app(char_type* n) { m_arg_ptrs.front() = n; }
            void  push_back(char_type* a) { m_arg_ptrs. back() = a; m_arg_ptrs.push_back(0); }
            
            char_type* const* ptr() const { return &m_arg_ptrs[0]; }
        
        private:
            typedef std::vector<char_type*> arg_ptrs_vec;
            
            arg_ptrs_vec m_arg_ptrs;
        };

        path           m_exe;
        arg_ptrs       m_arg_ptrs;
        char_type**    m_env_vars_ptrs;
        pid_type       m_pid;
        int            m_errno;
        
    private:
                   executor(const executor&);
        executor& operator=(const executor&);
    };

}}}

#undef BOOST_PROCESS_EXECUTOR_METHOD

#endif
