
// 
// Copyright (c) 2011 Jeff Flinn
// 
// Distributed under the Boost Software License, Version 1.0. (See accompanying 
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt) 
// 

#ifndef BOOST_PROCESS_WINDOWS_INITIALIZER_ARGS_HPP
#define BOOST_PROCESS_WINDOWS_INITIALIZER_ARGS_HPP

#include <boost/process/config.hpp>
#include <boost/process/windows/initializers/initializer.hpp>
#include <boost/lexical_cast.hpp>

#include <vector>

namespace boost { namespace process { namespace windows {

    struct executor;

    struct arg : public initializer 
    {
        typedef wchar_t        char_type;
        typedef std::wstring string_type;

        template<typename T> arg(const T& t) : m_arg(boost::lexical_cast<std::wstring>(t)) {}

        void  pre_create(executor& e) const 
        { 
            e.m_args.push_back(m_arg); 
        } 

        string_type m_arg;
    };

    struct args : public initializer 
    {
        typedef std::vector<arg> args_type;

        template<typename T> args(const T& t) : m_args(1, arg(t)) {}

        args& operator()(const arg& a)
        {
            m_args.push_back(a);

            return *this;
        }

        void  pre_create(executor& e) const 
        { 
            for_each(m_args.begin(), m_args.end(), initializer::lazy::pre_create(e));
        } 

        args_type m_args;
    };

}}}

#endif // BOOST_PROCESS_WINDOWS_INITIALIZER_ARGS_HPP