// 
// Copyright (c) 2011 Jeff Flinn
// 
// Distributed under the Boost Software License, Version 1.0. (See accompanying 
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt) 
// 

#include <boost/process/executor.hpp> 
#include <boost/process/initializers/paths.hpp>
#include <boost/process/initializers/args.hpp>
#include <boost/process/initializers/throw_on_launch_error.hpp>
#include <boost/process/initializers/std_io.hpp>
#include <boost/process/file_descriptor_ray.hpp>

#include <boost/filesystem/path.hpp>
#include <boost/fusion/container/vector/vector.hpp>
#include <boost/fusion/container/generation/make_vector.hpp>
#include <boost/iostreams/stream.hpp>
#include <boost/thread/thread.hpp>

#include <iostream>

int main() 
{
    namespace bp  = boost::process;
    namespace fus = boost::fusion;

    typedef bp::io_initializer::sink_type       sink_type;
    typedef boost::iostreams::stream<sink_type> sink_stream;
    typedef bp::file_descriptor_ray             ray_type;

    sink_type     s("C:\\TEMP\\log.txt");
    sink_stream log(s, std::ios_base::out|std::ios_base::binary);

    ray_type    ray1;
    sink_stream to_child(ray1.m_sink, std::ios_base::out|std::ios_base::binary);

    to_child << "testing_input_to_child " << std::endl;
    log      << "- before exec:"          << std::endl;

    try
    {
        bp::executor().exec(fus::make_vector
        ( bp::paths                ("C:\\TEMP\\some.exe")
        , bp::args                 ("C:\\TEMP\\file.txt")
        , bp::throw_on_launch_error()
        , bp::std_out_to_path      ("C:\\TEMP\\y.txt")
        , bp::std_err_to           (s)
        , bp::std_in_from          (ray1)
        ));
            
        boost::xtime xt;
        boost::xtime_get(&xt, boost::TIME_UTC);
        xt.sec += 1;

        boost::this_thread::sleep(xt);
    }
    catch(const std::exception& e)
    {
        std::string s = e.what();

        std::cout << s;
    }

    return 0; 
} 
